import doctest
from insights.parsers import dmsetup
from insights.parsers.dmsetup import DmsetupInfo, SetupInfo
from insights.parsers.dmsetup import DmsetupStatus, SetupStatus
from insights.tests import context_wrap

DMSETUP_INFO_1 = """
Name               Maj Min Stat Open Targ Event  UUID
VG00-tmp           253   8 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4
VG00-var_tmp       253   4 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxXOT2ZNHpEmJy2g2FpmXfAH1chG4Utm4Q
VG00-home          253   3 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxCqXOnbGe2zjhX923dFiIdl1oi7mO9tXp
VG00-var_log       253   9 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxlycWK5qprImfYnbkZLNiFZ5Lc6rJq04Z
VG00-usr           253   2 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxJqQ6DLofdR0uWkTnlpkRnFShO3PgqhCT
VG00-var           253   6 L--w    1    2      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxicvyvt67113nTb8vMlGfgdEjDx0LKT2O
VG00-swap          253   1 L--w    2    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax3Ll2XhOYZkylx1CjOQi7G4yHgrIOsyqG
VG00-root          253   0 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxKpnAKYhrYMYMNMwjegkW965bUgtJFTRY
VG00-var_log_audit 253   5 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxwQ8R0XWJRm86QX3befq1cHRy47Von6ZW
VG_DB-vol01        253  10 L--w    1    2      0 LVM-dgoBx4rat9aLu3sg1k95D7YwUT7YnFddgcZSaU2sjHZMVBlwcNwDjmmGGtfXeIZs
VG00-opt           253   7 L--w    1    4      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxIiCYm5hcvgQdXynPGBfHQLrtE3sqUKT2
""".strip()

DMSETUP_INFO_2 = """
Name               Maj Min Stat Open Targ Event  UUID
VG00-tmp           253   8 xIsr    1    1      a LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4
""".strip()


def test_dmsetup_info():
    r = DmsetupInfo(context_wrap(DMSETUP_INFO_1))
    assert len(r) == 11
    assert len(r[0]) == 8
    assert r[0]['UUID'] == 'LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4'
    assert r[-1]['Stat'] == 'L--w'
    assert r.names == ['VG00-tmp', 'VG00-var_tmp', 'VG00-home', 'VG00-var_log',
                       'VG00-usr', 'VG00-var', 'VG00-swap', 'VG00-root',
                       'VG00-var_log_audit', 'VG_DB-vol01', 'VG00-opt']
    assert r.names == [dm['Name'] for dm in r]
    assert len(r.by_uuid) == 11
    assert r.info[0] == SetupInfo(
        name='VG00-tmp',
        major=253,
        minor=8,
        open=1,
        segments=1,
        events=0,
        live_table=True,
        inactive_table=False,
        suspended=False,
        readonly=False,
        uuid='LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4'
    )
    assert r.info[-1] == SetupInfo(
        name='VG00-opt',
        major=253,
        minor=7,
        open=1,
        segments=4,
        events=0,
        live_table=True,
        inactive_table=False,
        suspended=False,
        readonly=False,
        uuid='LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxIiCYm5hcvgQdXynPGBfHQLrtE3sqUKT2'
    )


def test_dmsetup_setupinfo():
    r = DmsetupInfo(context_wrap(DMSETUP_INFO_2))
    assert r.info[0] == SetupInfo(
        name='VG00-tmp',
        major=253,
        minor=8,
        open=1,
        segments=1,
        events=None,
        live_table=False,
        inactive_table=True,
        suspended=True,
        readonly=True,
        uuid='LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4'
    )


DMSETUP_STATUS_1 = """
rootvg-tanlv: 0 6291456 linear
rootvg-ssnap: 0 16384000 snapshot 1560768/5120000 6088
rootvg-optvolapp: 0 8192000 snapshot-origin
docker-253:10-1234567-0df13579: 0 20971520 thin 1922048 20971519
docker-253:10-4254621-0496628a: 0 20971520 thin 1951744 20971519
docker-253:10-4254621-d392682f: 0 20971520 thin 7106560 20971519
rootvg-docker--pool: 0 129548288 thin-pool 1 20/49152 38/126512 - rw no_discard_passdown queue_if_no_space -
rootvg-tmpvol: 0 2048000 linear
rootvg-varvol: 0 18874368 snapshot Invalid
rootvg-optvol: 0 8192000 snapshot 616408/5120000 2408
rootvg-varvol-cow: 0 5120000 linear
appsvg-lvapps_docker: 0 104857600 thin-pool 441 697/2048 20663/102400 - rw no_discard_passdown queue_if_no_space -
""".strip()

DMSETUP_STATUS_2 = """
rootvg-tanlv: 0 6291456
rootvg-ssnap: 0 16384000 unknown-type
rootvg-docker--pool: 0 129548288 thin-pool 1 20/49152 38/126512 - rw no_discard_passdown queue_if_no_space
rootvg-optvol: 0 8192000 snapshot-origin
docker-253:10-4254621-d392682f: 0 20971520 thin 7106560 20971519
docker-253:10-1234567-0df13579: 0 20971520 thin 1922048
""".strip()

DMSETUP_STATUS_3 = """
No devices found
""".strip()


def test_dmsetup_status():
    r = DmsetupStatus(context_wrap(DMSETUP_STATUS_1))
    assert len(r) == 12
    assert len(r[0]) == 6
    assert r[0].device_name == 'rootvg-tanlv'
    assert r[0].start_sector == '0'
    assert r[0].num_sectors == '6291456'
    assert r[0].target_type == 'linear'
    assert r[0].target_args is None
    assert r[0].parsed_args is None
    assert r.names == ['rootvg-tanlv', 'rootvg-ssnap', 'rootvg-optvolapp',
            'docker-253:10-1234567-0df13579', 'docker-253:10-4254621-0496628a',
            'docker-253:10-4254621-d392682f', 'rootvg-docker--pool',
            'rootvg-tmpvol', 'rootvg-varvol', 'rootvg-optvol',
            'rootvg-varvol-cow', 'appsvg-lvapps_docker']

    assert len(r.by_name) == len([dev.device_name for dev in r])
    assert r[0] == SetupStatus(
            device_name='rootvg-tanlv', start_sector='0',
            num_sectors='6291456', target_type='linear',
            target_args=None, parsed_args=None,
    )
    assert r[-1] == SetupStatus(
            device_name='appsvg-lvapps_docker', start_sector='0',
            num_sectors='104857600', target_type='thin-pool',
            target_args='441 697/2048 20663/102400 - rw no_discard_passdown queue_if_no_space -',
            parsed_args={
                'transaction_id': '441',
                'used_metadata_blocks': '697',
                'total_metadata_blocks': '2048',
                'used_data_blocks': '20663',
                'total_data_blocks': '102400',
                'held_metadata_root': '-',
                'opts': ['rw', 'no_discard_passdown', 'queue_if_no_space', '-'],
                'metadata_low_watermark': None
            })
    assert r[3] == SetupStatus(
            device_name='docker-253:10-1234567-0df13579', start_sector='0',
            num_sectors='20971520', target_type='thin', target_args='1922048 20971519',
            parsed_args={'nr_mapped_sectors': '1922048', 'highest_mapped_sector': '20971519'}
    )
    assert r[-3] == SetupStatus(
            device_name='rootvg-optvol', start_sector='0', num_sectors='8192000',
            target_type='snapshot', target_args='616408/5120000 2408',
            parsed_args={'sectors_allocated': '616408', 'total_sectors': '5120000', 'metadata_sectors': '2408'}
    )
    assert r[-4] == SetupStatus(
            device_name='rootvg-varvol', start_sector='0', num_sectors='18874368',
            target_type='snapshot', target_args='Invalid', parsed_args=None
    )
    assert r.unparseable_lines == ['rootvg-varvol: 0 18874368 snapshot Invalid']

    r = DmsetupStatus(context_wrap(DMSETUP_STATUS_2))
    assert len(r) == 5
    assert r.unparseable_lines == [
            'rootvg-tanlv: 0 6291456',
            'rootvg-docker--pool: 0 129548288 thin-pool 1 20/49152 38/126512 - rw no_discard_passdown queue_if_no_space',
            'docker-253:10-1234567-0df13579: 0 20971520 thin 1922048']

    r = DmsetupStatus(context_wrap(DMSETUP_STATUS_3))
    assert len(r) == 0
    assert r.unparseable_lines == []


DMSETUP_EXAMPLES = """
Name               Maj Min Stat Open Targ Event  UUID
VG00-tmp           253   8 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4
VG00-home          253   3 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxCqXOnbGe2zjhX923dFiIdl1oi7mO9tXp
VG00-var           253   6 L--w    1    2      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxicvyvt67113nTb8vMlGfgdEjDx0LKT2O
VG00-swap          253   1 L--w    2    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax3Ll2XhOYZkylx1CjOQi7G4yHgrIOsyqG
VG00-root          253   0 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxKpnAKYhrYMYMNMwjegkW965bUgtJFTRY
VG00-var_log_audit 253   5 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxwQ8R0XWJRm86QX3befq1cHRy47Von6ZW
""".strip()


def test_examples():
    env = {
        'setup_info': DmsetupInfo(context_wrap(DMSETUP_EXAMPLES)),
        'dmsetup_status': DmsetupStatus(context_wrap(DMSETUP_STATUS_1))
    }
    failed, total = doctest.testmod(dmsetup, globs=env)
    assert failed == 0



"""
Custom datasource for CVE-2021-35937, CVE-2021-35938, and CVE-2021-35939.
"""
import grp
import pwd
import signal

from collections import defaultdict

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """
    Local spec used only by the rpm_pkgs datasource.
    """
    rpm_args = simple_command(
        'rpm -qa --nosignature --qf="[%{=NAME}; %{=NEVRA}; %{FILENAMES}; %{FILEMODES:perms}; %{FILEUSERNAME}; %{FILEGROUPNAME}; %{=VENDOR}\n]"',
        signum=signal.SIGTERM
    )


def get_shells():
    """
    Returns all full pathnames of valid login shells without nologins.
    """
    with open("/etc/shells") as file:
        return set(line.strip() for line in file if "nologin" not in line)


def get_users():
    """
    Returns all users with shell specified in get_shells() except for root.
    """
    shells = get_shells()
    users = set()

    for user in pwd.getpwall():
        name = user[0]
        shell = user[6]

        if name == "root" or (shell not in shells):
            continue
        users.add(name)
    return users


def get_groups(users):
    """
    Returns all groups for users specified in get_users().
    Every user has at least one group with its name.
    """
    groups = set()

    for group in grp.getgrall():
        group_name = group[0]
        user_list = group[3]

        if group_name in users:
            groups.add(group_name)
        else:
            for user in user_list:
                if user.strip() in users:
                    groups.add(group_name)
                    break
    return groups


@datasource(LocalSpecs.rpm_args, HostContext)
def pkgs_with_writable_dirs(broker):
    r"""
    Custom datasource for CVE-2021-35937, CVE-2021-35938, and CVE-2021-35939.

    It collects packages from the ``rpm -qa --nosignature --qf="[%{=NAME}; %{=NEVRA}; %{FILENAMES};
    %{FILEMODES:perms}; %{FILEUSERNAME}; %{FILEGROUPNAME}; %{=VENDOR}\n]" command``.

    The output is a sorted list of all packages, which have at least one directory with files
    inside, and this directory is writable by a specific user/group or the others.

    Raises:
        SkipComponent: Raised if no data is available

    Returns:
        List[str]: Sorted list of strings, where every string contains `name`, `nevra`
        and `vendor` for a given package, and the pipe is used as a separator.
        E.g. ["httpd-core|httpd-core-2.4.53-7.el9.x86_64|Red Hat, Inc."]
    """
    content = broker[LocalSpecs.rpm_args].content

    if not content or "command not found" in content[0]:
        raise SkipComponent

    users = get_users()
    groups = get_groups(users)

    dir_package = defaultdict(set)
    dirs = set()

    for line in content:
        pkg_name, nevra, path_name, perms, user, group, vendor = line.split("; ")

        if perms[0] == "d":
            user_w = user in users and perms[2] == "w"
            group_w = group in groups and perms[5] == "w"
            others_w = perms[8] == "w"

            if user_w or group_w or others_w:
                # Stores a writeable directory with its package
                dir_package[path_name].add("{0}|{1}|{2}".format(pkg_name, nevra, vendor))
        else:
            # Stores a file directory
            dirs.add(path_name.rsplit('/', 1)[0])

    # Stores a package if its directory has files inside
    packages = set()
    for dir_path in dir_package:
        if dir_path in dirs:
            packages.update(dir_package[dir_path])

    if packages:
        return DatasourceProvider(
            content=sorted(packages), relative_path="insights_commands/rpm_pkgs"
        )



"""
AutoFSConf - file ``/etc/autofs.conf``
======================================
"""

from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.autofs_conf)
class AutoFSConf(IniConfigFile):
    """
    The `/etc/autofs.conf` file is in a standard '.ini' format, and this parser
    uses the IniConfigFile base class to read this.

    Sample configuration::

        [ autofs ]
        timeout = 300
        browse_mode = no
        mount_nfs_default_protocol = 4

        [ amd ]
        dismount_interval = 300

    Examples:
        >>> type(config)
        <class 'insights.parsers.autofs_conf.AutoFSConf'>
        >>> config.sections()
        ['autofs', 'amd']
        >>> config.has_option('amd', 'dismount_interval')
        True
        >>> config.get('amd', 'dismount_interval')
        '300'
        >>> config.getint('autofs', 'timeout')
        300
        >>> config.getboolean('autofs', 'browse_mode')
        False
    """
    pass



"""
Custom datasources for leapp
"""
import json

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_file
from insights.specs import Specs


class LocalSpecs(Specs):
    leapp_report_raw = simple_file("/var/log/leapp/leapp-report.json")
    """ Returns the contents of the file ``/var/log/leapp/leapp-report.json`` """


@datasource(LocalSpecs.leapp_report_raw, HostContext)
def leapp_report(broker):
    """
    This datasource get useful information from ``leapp-report.json``.

    .. note::
        Since this file may contain sensitive information, this datasource does
        filter and only keep required data.

    Returns:
        str: JSON string after get the required data.

    Raises:
        SkipComponent: When the path does not exist, nothing is collected,
                       or any exception occurs.
    """
    relative_path = 'insights_commands/leapp_report'
    valid_keys = {
        'inhibitor': ['title', 'summary', ('detail', 'remediations')]
    }
    try:
        content = broker[LocalSpecs.leapp_report_raw].content
        report = json.loads('\n'.join(content))
        if isinstance(report, dict):
            results = []
            for entry in report.get('entries', []):
                groups = entry.get('groups', entry.get('flags', []))
                for flag, keys in valid_keys.items():
                    if flag in groups:
                        ret = dict(groups=groups)
                        for key in keys:
                            if isinstance(key, tuple):
                                tmp = entry
                                for k in key:
                                    tmp = tmp[k]
                                ret[key[-1]] = tmp
                            else:
                                ret[key] = entry[key]
                        results.append(ret)
            if results:
                return DatasourceProvider(content=json.dumps(results), relative_path=relative_path)
        raise SkipComponent("Invalid format")
    except Exception as e:
        raise SkipComponent(e)
    raise SkipComponent



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import pmlog_summary
from insights.parsers.pmlog_summary import PmLogSummary
from insights.tests import context_wrap

PMLOG = """
mem.util.used  3133919.812 Kbyte
mem.physmem  3997600.000 Kbyte
kernel.all.cpu.user  0.003 none
kernel.all.cpu.sys  0.004 none
kernel.all.cpu.nice  0.000 none
kernel.all.cpu.steal  0.000 none
kernel.all.cpu.idle  3.986 none
kernel.all.pressure.io.full.avg ["10 second"] 0.001 none
kernel.all.pressure.cpu.some.avg ["1 minute"] 14.942 none
kernel.all.pressure.memory.full.avg ["5 minute"] 0.002 none
disk.all.total  0.252 count / sec
disk.dev.total ["vda"] 0.016 count / sec
disk.dev.total ["vdb"] 0.445 count / sec
disk.dev.total ["vdc"] 2.339 count / sec
"""

PMLOG_EMPTY = """
"""


def test_pmlog_summary():
    pmlog_summary = PmLogSummary(context_wrap(PMLOG))
    assert len(pmlog_summary) == 3
    assert len(pmlog_summary['mem']) == 2
    assert pmlog_summary['mem']['util']['used'] == {'val': 3133919.812, 'units': 'Kbyte'}
    assert pmlog_summary['mem']['physmem'] == {'val': 3997600.0, 'units': 'Kbyte'}
    assert pmlog_summary['disk']['all']['total'] == {'val': 0.252, 'units': 'count / sec'}
    assert pmlog_summary['kernel']['all']['pressure']['cpu']['some']['avg']['1 minute'] == {'val': 14.942, 'units': 'none'}
    assert 'not.present' not in pmlog_summary
    assert pmlog_summary['kernel']['all']['cpu'] == {
        'user': {'val': 0.003, 'units': 'none'},
        'sys': {'val': 0.004, 'units': 'none'},
        'nice': {'val': 0.0, 'units': 'none'},
        'steal': {'val': 0.0, 'units': 'none'},
        'idle': {'val': 3.986, 'units': 'none'},
    }
    assert pmlog_summary['disk']['dev']['total'] == {
        'vda': {'val': 0.016, 'units': 'count / sec'},
        'vdb': {'val': 0.445, 'units': 'count / sec'},
        'vdc': {'val': 2.339, 'units': 'count / sec'},
    }


def test_pmlog_summmary():
    with pytest.raises(SkipComponent):
        PmLogSummary(context_wrap(PMLOG_EMPTY))


def test_doc_examples():
    env = {
        'pmlog_summary': PmLogSummary(context_wrap(PMLOG))
    }
    failed, _ = doctest.testmod(pmlog_summary, globs=env)
    assert failed == 0



"""
subscription-manager commands
=============================

Parsers for parsing output of the ``subscription-manager`` commands.

SubscriptionManagerFacts - command ``subscription-manager facts``
-----------------------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

add_filter(Specs.subscription_manager_facts, ['instance_id'])


@parser(Specs.subscription_manager_facts)
class SubscriptionManagerFacts(CommandParser, dict):
    """
    Class for parsing the output of `subscription-manager facts` command.

    Typical output of the command is::

        aws_instance_id: 567890567890
        network.ipv6_address: ::1
        uname.sysname: Linux
        uname.version: #1 SMP PREEMPT Fri Sep 2 16:07:40 EDT 2022
        virt.host_type: rhev, kvm
        virt.is_guest: True

    Examples:
        >>> type(rhsm_facts)
        <class 'insights.parsers.subscription_manager.SubscriptionManagerFacts'>
        >>> rhsm_facts['aws_instance_id']
        '567890567890'
    """
    def parse_content(self, content):
        for line in content:
            if ': ' not in line:
                raise ParseException('Incorrect line: {0}'.format(line))
            key, val = [_l.strip() for _l in line.split(': ', 1)]
            self[key] = val

        if len(self) == 0:
            raise SkipComponent



import doctest
import pytest

from insights import SkipComponent
from insights.parsers.cryptsetup_luksDump import LuksDump
from insights.parsers import luksmeta
from insights.combiners.cryptsetup import LuksDevices
import insights.combiners.cryptsetup
from insights.tests import context_wrap

LUKS1_DUMP = """LUKS header information for luks1

Version:       	1
Cipher name:   	aes
Cipher mode:   	xts-plain64
Hash spec:     	sha256
Payload offset:	4096
MK bits:       	512
MK digest:     	ca fe ba be df 8c c4 b4 b8 0a cc dd 98 b5 d8 64 3a 95 3e 9e 
MK salt:       	ca fe ba be 04 3b 77 d8 ff 08 1e 0a 41 68 45 a5 
               	ca fe ba be 7b 3f a9 69 9c 9b 51 24 58 47 8d a2 
               	ca fe ba be 7b 3f a9 69 9c 9b 51 24 58 47 8d a2 
               	ca fe ba be 7b 3f a9 69 9c 9b 51 24 58 47 8d a2 
               	ca fe ba be 7b 3f a9 69 9c 9b 51 24 58 47 8d a2 
               	ca fe ba be 7b 3f a9 69 9c de ad be ef 
MK iterations: 	106562
UUID:          	263902da-5f0c-43a9-82eb-cc6f14d90448

Key Slot 0: ENABLED
	Iterations:         	2099250
	Salt:               	de ad be ef 
	Salt:               	ca fe ba be a1 f3 ae cb 4a 3f f0 2d de ad be ef 
	                      	de ad be ef 
	Key material offset:	8
	AF stripes:            	4000
Key Slot 1: ENABLED
	Iterations:         	1987820
	Salt:               	ca fe ba be f2 b7 7d f3 29 c2 c8 80 de ad be ef 
	                      	ca fe ba be 9f a1 87 07 c6 4f aa cd de ad be ef 
	                      	ca fe ba be 9f a1 87 07 c6 4f aa de ad be ef 
	Key material offset:	512
	AF stripes:            	4000
Key Slot 2: ENABLED
	Iterations:         	2052006
	Salt:               	ca fe ba be 47 94 e7 40 22 c1 bb 4a de ad be ef 
	                      	ca fe ba be 52 e8 8d 70 b2 1e 9d 47 de ad be ef 
	Key material offset:	1016
	AF stripes:            	4000
Key Slot 3: DISABLED
Key Slot 4: DISABLED
Key Slot 5: DISABLED
Key Slot 6: DISABLED
Key Slot 7: DISABLED
"""  # noqa

LUKS2_DUMP = """LUKS header information
Version:       	2
Epoch:         	6
Metadata area: 	16384 [bytes]
Keyslots area: 	16744448 [bytes]
UUID:          	cfbcc942-e06b-4c4a-952f-e9c9b2011c27
Label:         	(no label)
Subsystem:     	(no subsystem)
Flags:       	(no flags)

Data segments:
  0: crypt
	offset: 16777216 [bytes]
	length: (whole device)
	cipher: aes-xts-plain64
	sector: 4096 [bytes]

Keyslots:
  0: luks2
	Key:        512 bits
	Priority:   normal
	Cipher:     aes-xts-plain64
	Cipher key: 512 bits
	PBKDF:      argon2id
	Time cost:  7
	Memory:     1048576
	Threads:    4
	Salt:       ca fe ba be fe 1c 90 d8 2a 35 b2 b2 de ad be ef 
	            ca fe ba be b2 dd 45 9e ed 9a 33 b2 de ad be ef 
                    de ad be ef 
	AF stripes: 4000
	AF hash:    sha256
	Area offset:32768 [bytes]
	Area length:258048 [bytes]
	Digest ID:  0
  1: luks2
	Key:        512 bits
	Priority:   normal
	Cipher:     aes-xts-plain64
	Cipher key: 512 bits
	PBKDF:      argon2id
	Time cost:  7
	Memory:     1048576
	Threads:    4
	Salt:       ca fe ba be c1 94 15 86 2a e9 26 f8 de ad be ef 
	            ca fe ba be 05 2d 80 c9 56 e8 4d 6f de ad be ef 
	AF stripes: 4000
	AF hash:    sha256
	Area offset:290816 [bytes]
	Area length:258048 [bytes]
	Digest ID:  0
  2: luks2
	Key:        512 bits
	Priority:   normal
	Cipher:     aes-xts-plain64
	Cipher key: 512 bits
	PBKDF:      pbkdf2
	Hash:       sha512
	Iterations: 1000
	Salt:       ca fe ba be d7 8f a6 de a0 cb a4 d1 de ad be ef 
	            ca fe ba be fb 53 43 06 e8 83 90 93 de ad be ef 
	AF stripes: 4000
	AF hash:    sha512
	Area offset:548864 [bytes]
	Area length:258048 [bytes]
	Digest ID:  0
Tokens:
  0: systemd-tpm2
        tpm2-pcrs:  7
        tpm2-bank:  sha256
        tpm2-primary-alg:  ecc
        tpm2-blob:  00 9e 00 20 bd 97 78 70 3f 3a 5b 93 d4 8f dc ed
                    10 16 b2 ce f5 f7 a2 c8 63 f6 19 12 63 7a f2 94
                    26 f1 b6 1b 00 10 2e 36 26 c1 3b f7 1e 8d 86 55
        tpm2-policy-hash:
                    df 06 80 28 e7 67 b1 d0 34 f4 de 1b 8e ac 33 5a
                    df 06 80 28 e7 67 b1 d0 34 f4 de 1b 8e ac 33 5a
	Keyslot:    2
Digests:
  0: pbkdf2
	Hash:       sha256
	Iterations: 129774
	Salt:       ca fe ba be e0 65 83 82 35 03 29 56 de ad be ef 
	            ca fe ba be de 69 39 97 d5 b3 ac c4 de ad be ef 
                    de ad be ef 
	Digest:     ca fe ba be 9d 46 9b 0f 3a 0f 57 13 de ad be ef 
	            ca fe ba be ed 7d 09 2c 3d b6 fa f4 de ad be ef 
"""  # noqa

LUKSMETA_OUTPUT = """0   active empty
1   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
2   active empty
3   active empty
4 inactive empty
5   active empty
6   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
7   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
"""  # noqa

luks1_device = LuksDump(context_wrap(LUKS1_DUMP))
luks2_device = LuksDump(context_wrap(LUKS2_DUMP))
uuid = luks1_device.dump["header"]["UUID"]
luksmeta_parsed = luksmeta.LuksMeta(context_wrap(LUKSMETA_OUTPUT, path="/insights_commands/cryptsetup_luksDump_--disable-external-tokens_.dev.disk.by-uuid." + uuid))
luksmeta_parsed_no_uuid = luksmeta.LuksMeta(context_wrap(LUKSMETA_OUTPUT))


def test_luks_devices_combiner():
    with pytest.raises(SkipComponent):
        luks_devices = LuksDevices([], None)

    luks_devices = LuksDevices([luks1_device, luks2_device], None)
    for device in luks_devices:
        assert "luksmeta" not in device

    luks_devices = LuksDevices([luks1_device, luks2_device], [])
    for device in luks_devices:
        assert "luksmeta" not in device

    luks_devices = LuksDevices([luks1_device, luks2_device], [luksmeta_parsed_no_uuid])
    for device in luks_devices:
        assert "luksmeta" not in device

    luks_devices = LuksDevices([luks1_device, luks2_device], [luksmeta_parsed])
    for device in luks_devices:
        if device["header"]["UUID"] == uuid:
            assert "luksmeta" in device
        else:
            assert "luksmeta" not in device


def test_doc_examples():
    env = {
            'luks_devices': LuksDevices([luks1_device, luks2_device], [luksmeta_parsed])
          }
    failed, total = doctest.testmod(insights.combiners.cryptsetup, globs=env)
    assert failed == 0



import doctest

from insights.parsers import designate_conf
from insights.tests import context_wrap

DESIGNATE_CONF = """
[DEFAULT]
state_path=/var/lib/designate
debug=True
log_dir=/var/log/designate

[keystone_authtoken]
www_authenticate_uri=http://localhost:5000
project_name=service
project_domain_name=Default

[oslo_messaging_notifications]
driver=messagingv2
"""


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        designate_conf,
        globs={'conf': designate_conf.DesignateConf(context_wrap(DESIGNATE_CONF))}
    )
    assert failed_count == 0


def test_designate_conf():
    dconf = designate_conf.DesignateConf(context_wrap(DESIGNATE_CONF))
    assert dconf is not None
    assert list(dconf.sections()) == ['keystone_authtoken', 'oslo_messaging_notifications']
    assert dconf.defaults() == {
        'debug': 'True',
        'state_path': '/var/lib/designate',
        'log_dir': '/var/log/designate'}
    assert dconf.get('keystone_authtoken', 'project_name') == 'service'
    assert dconf.has_option('oslo_messaging_notifications', 'driver')
    assert not dconf.has_option('yabba', 'dabba_do')
    assert dconf.get('DEFAULT', 'debug') == 'True'



import pytest

from insights.core.exceptions import ParseException
from insights.parsers.getcert_list import CertList
from insights.tests import context_wrap

CERT_LIST_1 = """
Number of certificates and requests being tracked: 8.
Request ID '20150522133327':
        status: MONITORING
        stuck: no
        key pair storage: type=NSSDB,location='/etc/dirsrv/slapd-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB',pinfile='/etc/dirsrv/slapd-EXAMPLE-COM/pwdfile.txt'
        certificate: type=NSSDB,location='/etc/dirsrv/slapd-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB'
        CA: IPA
        issuer: CN=Certificate Authority,O=EXAMPLE.COM
        subject: CN=ldap.example.com,O=EXAMPLE.COM
        expires: 2017-05-22 13:33:27 UTC
        key usage: digitalSignature,nonRepudiation,keyEncipherment,dataEncipherment
        eku: id-kp-serverAuth,id-kp-clientAuth
        pre-save command:
        post-save command: /usr/lib64/ipa/certmonger/restart_dirsrv EXAMPLE-COM
        track: yes
        auto-renew: yes
Request ID '20150522133549':
        status: MONITORING
        stuck: no
        key pair storage: type=NSSDB,location='/etc/httpd/alias',nickname='Server-Cert',token='NSS Certificate DB',pinfile='/etc/httpd/alias/pwdfile.txt'
        certificate: type=NSSDB,location='/etc/httpd/alias',nickname='Server-Cert',token='NSS Certificate DB'
        CA: IPA
        issuer: CN=Certificate Authority,O=EXAMPLE.COM
        subject: CN=ldap.example.com,O=EXAMPLE.COM
        expires: 2017-05-22 13:35:49 UTC
        key usage: digitalSignature,nonRepudiation,keyEncipherment,dataEncipherment
        eku: id-kp-serverAuth,id-kp-clientAuth
        pre-save command:
        post-save command: /usr/lib64/ipa/certmonger/restart_httpd
        track: yes
        auto-renew: yes
"""


def test_getcert_1():
    certs = CertList(context_wrap(CERT_LIST_1, path="sos_commands/ipa/ipa-getcert_list"))

    assert certs.num_tracked == 8
    # Number of actual requests that we see here.
    assert sorted(certs.requests) == sorted(['20150522133327', '20150522133549'])
    # Treated as a pseudo-dict
    assert len(certs) == 2
    assert '20150522133327' in certs
    assert certs['20150522133327']['status'] == 'MONITORING'
    assert certs['20150522133327']['stuck'] == 'no'
    assert certs['20150522133327']['key pair storage'] == "type=NSSDB,location='/etc/dirsrv/slapd-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB',pinfile='/etc/dirsrv/slapd-EXAMPLE-COM/pwdfile.txt'"
    assert certs['20150522133327']['certificate'] == "type=NSSDB,location='/etc/dirsrv/slapd-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB'"
    assert certs['20150522133327']['CA'] == 'IPA'
    assert certs['20150522133327']['issuer'] == 'CN=Certificate Authority,O=EXAMPLE.COM'
    assert certs['20150522133327']['subject'] == 'CN=ldap.example.com,O=EXAMPLE.COM'
    assert certs['20150522133327']['expires'] == '2017-05-22 13:33:27 UTC'
    assert certs['20150522133327']['key usage'] == 'digitalSignature,nonRepudiation,keyEncipherment,dataEncipherment'
    assert certs['20150522133327']['eku'] == 'id-kp-serverAuth,id-kp-clientAuth'
    assert certs['20150522133327']['pre-save command'] == ''
    assert certs['20150522133327']['post-save command'] == '/usr/lib64/ipa/certmonger/restart_dirsrv EXAMPLE-COM'
    assert certs['20150522133327']['track'] == 'yes'
    assert certs['20150522133327']['auto-renew'] == 'yes'

    # keyword search tests
    assert certs.search(stuck='no') == [certs['20150522133327'], certs['20150522133549']]


CERT_BAD_1 = """
Number of certificates and requests being tracked: d.
"""

CERT_BAD_2 = """
Number of certificates and requests being tracked: 8.
Request ID '20150522133327':
        status: MONITORING
        stuck: no
Request ID '20150522133327':
        status: MONITORING
        stuck: no
"""


def test_getcert_exceptions():
    with pytest.raises(ParseException) as exc:
        assert CertList(context_wrap(CERT_BAD_1)) is None
    assert 'Incorrectly formatted number of certificates and requests' in str(exc)
    with pytest.raises(ParseException) as exc:
        assert CertList(context_wrap(CERT_BAD_2)) is None
    assert "Found duplicate request ID '20150522133327'" in str(exc)


CERT_BAD_3 = """
Number of certificates and requests being tracked: 2.
Request ID '20150522133327':
        status: MONITORING
        invalid object
"""


def test_getcert_coverage():
    certs = CertList(context_wrap(CERT_BAD_3))
    assert certs
    assert certs.num_tracked == 2
    assert '20150522133327' in certs
    # Lines without a colon get ignored
    assert certs['20150522133327'] == {'status': 'MONITORING'}



"""
YumLog - file ``/var/log/yum.log``
==================================

This module provides parsing for the ``/var/log/yum.log`` file.
The ``YumLog`` class implements parsing for the file, which looks like::

    May 13 15:54:49 Installed: libevent-2.0.21-4.el7.x86_64
    May 13 15:54:49 Installed: tmux-1.8-4.el7.x86_64
    May 23 18:06:24 Installed: wget-1.14-10.el7_0.1.x86_64
    May 23 18:10:05 Updated: 1:openssl-libs-1.0.1e-51.el7_2.5.x86_64
    May 23 18:10:05 Installed: 1:perl-parent-0.225-244.el7.noarch
    May 23 18:10:05 Installed: perl-HTTP-Tiny-0.033-3.el7.noarch
    May 23 16:09:09 Erased: redhat-access-insights-batch
    May 23 18:10:05 Installed: perl-podlators-2.5.1-3.el7.noarch
    May 23 18:10:05 Installed: perl-Pod-Perldoc-3.20-4.el7.noarch
    May 23 18:10:05 Installed: 1:perl-Pod-Escapes-1.04-286.el7.noarch
    May 23 18:10:06 Installed: perl-Text-ParseWords-3.29-4.el7.noarch

The information is stored as a ``list`` of ``Entry`` objects, each of which
contains attributes for the position in the log, timestamp of the action,
the package's state in the system, and the affected package as an
``InstalledRpm``.

Note:
    The examples in this module may be executed with the following command:

    ``python -m insights.parsers.yumlog``

Examples:
    >>> content = '''
    ... May 23 18:06:24 Installed: wget-1.14-10.el7_0.1.x86_64
    ... Jan 24 00:24:00 Updated: glibc-2.12-1.149.el6_6.4.x86_64
    ... Jan 24 00:24:09 Updated: glibc-devel-2.12-1.149.el6_6.4.x86_64
    ... Jan 24 00:24:10 Updated: nss-softokn-3.14.3-19.el6_6.x86_64
    ... Jan 24 18:10:05 Updated: 1:openssl-libs-1.0.1e-51.el7_2.5.x86_64
    ... Jan 24 00:24:11 Updated: glibc-2.12-1.149.el6_6.4.i686
    ... May 23 16:09:09 Erased: redhat-access-insights-batch
    ... Jan 24 00:24:11 Updated: glibc-devel-2.12-1.149.el6_6.4.i686
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> yl = YumLog(context_wrap(content))
    >>> e = yl.present_packages.get('nss-softokn')
    >>> e.pkg.release
    '19.el6_6'
    >>> e = yl.present_packages.get('openssl-libs')
    >>> e.pkg.name
    'openssl-libs'
    >>> e.pkg.version
    '1.0.1e'
    >>> len(yl)
    8
    >>> indices = [e.idx for e in yl]
    >>> indices == range(len(yl))
    True
"""
from collections import defaultdict, namedtuple
from insights.core import Parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.parsers.installed_rpms import InstalledRpm
from insights.specs import Specs


Entry = namedtuple('Entry', field_names='idx timestamp state pkg')
"""namedtuple: Represents a line in ``/var/log/yum.log``."""


@parser(Specs.yum_log)
class YumLog(Parser):
    """Class for parsing /var/log/yum.log"""

    ERASED = 'Erased'
    """Package Erased"""

    INSTALLED = 'Installed'
    """Package Installed"""

    UPDATED = 'Updated'
    """Package Updated"""

    STATES = set([ERASED, INSTALLED, UPDATED])

    def parse_content(self, content):
        """Parses contents of each line in ``/var/log/yum.log``.

        Each line in the file contains 5 fields that are parsed into the
        attributes of ``Entry`` instances:

            - month
            - day
            - time
            - state
            - package

        The month, day, and time form the ``Entry.timestamp``.
        ``Entry.state`` contains the state of the package, one of ``ERASED``,
        ``INSTALLED``, or ``UPDATED``. ``Entry.pkg`` contains the ``InstalledRpm``
        instance corresponding to the parse package. ``Entry.idx`` is the
        zero-based line number of the ``Entry`` in the file.  It can be
        used to tell ordering of events.

        Parameters:
            content (list): Lines of ``/var/log/yum.log`` to be parsed.

        Raises:
            ParseException: if a line can't be parsed for any reason.
        """
        self.data = []
        self.pkgs = defaultdict(list)
        for idx, line in enumerate(get_active_lines(content)):
            if not any(s in line for s in self.STATES):
                continue
            try:
                line = line.replace(': 100', '')
                month, day, time, state, pkg = line.split()[:5]
                timestamp = ' '.join([month, day, time])
                state = state.rstrip(':')
                pkg = pkg.split(':')[-1].strip()
                if state == self.ERASED and "." not in pkg:
                    pkg = InstalledRpm({'name': pkg})
                else:
                    pkg = InstalledRpm.from_package(pkg)
                e = Entry(idx, timestamp, state, pkg)
                self.data.append(e)
                self.pkgs[pkg.name].append(e)
            except:
                raise ParseException('YumLog could not parse', line)

        self.pkgs = dict(self.pkgs)

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        for d in self.data:
            yield d

    def _packages_currently(self, states):
        result = {}
        for k, v in self.pkgs.items():
            e = v[-1]
            if e.state in states:
                result[k] = e

        return result

    @property
    def present_packages(self):
        """``list`` of latest ``Entry`` instances for installed packages."""
        return self._packages_currently([self.INSTALLED, self.UPDATED])



from insights.core.spec_factory import simple_file
from functools import partial
from insights.core.context import MustGatherContext
from insights.specs import Specs

simple_file = partial(simple_file, context=MustGatherContext)


class MustGatherArchiveSpecs(Specs):

    ceph_health_detail = simple_file("ceph/namespaces/openshift-storage/must_gather_commands/json_output/ceph_health_detail_--format_json-pretty")



"""
LsSystemdUnits - command ``/bin/ls -lanRL /etc/systemd /run/systemd /usr/lib/systemd /usr/local/lib/systemd /usr/local/share/systemd /usr/share/systemd``
=========================================================================================================================================================

This parser provides file listing for Systemd Units gathered from the ``/bin/ls -lanRL /etc/systemd
/run/systemd /usr/lib/systemd /usr/local/lib/systemd /usr/local/share/systemd /usr/share/systemd``
command.


The shortened sample output of the command is::

    /etc/systemd:
    total 40
    drwxr-xr-x.  4 0 0  150 Apr  4  2019 .
    drwxr-xr-x. 88 0 0 8192 Jun 28 06:56 ..
    -rw-r--r--.  1 0 0  615 Jun 22  2018 coredump.conf
    -rw-r--r--.  1 0 0 1027 Jun 22  2018 journald.conf
    -rw-r--r--.  1 0 0 1052 Apr  4  2019 logind.conf
    -rw-r--r--.  1 0 0  631 Feb 26  2019 resolved.conf
    drwxr-xr-x. 17 0 0 4096 Apr  4  2019 system
    -rw-r--r--.  1 0 0 1682 Feb 26  2019 system.conf
    drwxr-xr-x.  2 0 0    6 Feb 26  2019 user
    -rw-r--r--.  1 0 0 1130 Jun 22  2018 user.conf

    /etc/systemd/system:
    total 28
    drwxr-xr-x. 17 0 0 4096 Apr  4  2019 .
    drwxr-xr-x.  4 0 0  150 Apr  4  2019 ..
    drwxr-xr-x.  2 0 0   31 Apr  4  2019 basic.target.wants
    -rw-r--r--.  1 0 0 1341 Feb  8  2019 dbus-org.freedesktop.NetworkManager.service
    -rw-r--r--.  1 0 0  353 Feb  8  2019 dbus-org.freedesktop.nm-dispatcher.service
    -rw-r--r--.  1 0 0  238 Nov  7  2017 dbus-org.freedesktop.timedate1.service
    -rw-r--r--.  1 0 0  532 Jun 22  2018 default.target
    drwxr-xr-x.  2 0 0   32 Apr  4  2019 getty.target.wants
    drwxr-xr-x.  2 0 0 4096 Apr  4  2019 multi-user.target.wants
    drwxr-xr-x.  2 0 0   48 Apr  4  2019 network-online.target.wants
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 nfs-blkmap.service.requires
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 nfs-idmapd.service.requires
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 nfs-mountd.service.requires
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 nfs-server.service.requires
    drwxr-xr-x.  2 0 0   31 Apr  4  2019 remote-fs.target.wants
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 rpc-gssd.service.requires
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 rpc-statd-notify.service.requires
    drwxr-xr-x.  2 0 0   33 Apr  4  2019 rpc-statd.service.requires
    drwxr-xr-x.  2 0 0   51 Apr  4  2019 sockets.target.wants
    drwxr-xr-x.  2 0 0  151 Apr  4  2019 sysinit.target.wants
    -rw-r--r--.  1 0 0  583 Dec 17  2018 syslog.service
    crw-rw-rw-.  1 0 0 1, 3 Jun 28 06:56 systemd-timedated.service
    drwxr-xr-x.  2 0 0   34 Apr  4  2019 timers.target.wants

    /etc/systemd/system/basic.target.wants:
    total 8
    drwxr-xr-x.  2 0 0   31 Apr  4  2019 .
    drwxr-xr-x. 17 0 0 4096 Apr  4  2019 ..
    -rw-r--r--.  1 0 0  284 Nov  6  2018 microcode.service

Examples:
    >>> type(ls_systemd_units)
    <class 'insights.parsers.ls_systemd_units.LsSystemdUnits'>
    >>> ls_systemd_units.dirs_of("/etc/systemd")
    ['.', '..', 'system', 'user']
    >>> ls_systemd_units.files_of("/etc/systemd/system/basic.target.wants")
    ['microcode.service']
    >>> ls_systemd_units.specials_of("/etc/systemd/system")
    ['systemd-timedated.service']
    >>> ls_systemd_units.dir_contains("/etc/systemd/system", "default.target")
    True
    >>> ls_systemd_units.dir_entry("/etc/systemd/system", "syslog.service")["perms"]
    'rw-r--r--.'
"""

from insights.core import CommandParser, FileListing
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ls_systemd_units)
class LsSystemdUnits(CommandParser, FileListing):
    """
    Class for parsing the ``/bin/ls -lanRL /etc/systemd /run/systemd /usr/lib/systemd
    /usr/local/lib/systemd /usr/local/share/systemd /usr/share/systemd`` command.

    For more information, see the ``FileListing`` class.
    """
    pass



"""
saphostctrl - Commands ``saphostctrl``
======================================

Parsers included in this module are:

SAPHostCtrlInstances - Command ``saphostctrl -function GetCIMObject -enuminstances SAPInstance``
------------------------------------------------------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs


SAP_INST_FILTERS = [
        '******',
        'SID',
        'SystemNumber',
        'InstanceName',
        'InstanceType',
        'Hostname',
        'SapVersionInfo',
        'FullQualifiedHostname'
]
add_filter(Specs.saphostctl_getcimobject_sapinstance, SAP_INST_FILTERS)


@parser(Specs.saphostctl_getcimobject_sapinstance)
class SAPHostCtrlInstances(CommandParser, list):
    """
    This class provides processing for the output of the
    ``/usr/sap/hostctrl/exe/saphostctrl -function GetCIMObject -enuminstances SAPInstance``
    command on SAP systems.

    Sample output of the command::

        *********************************************************
         SID , String , D89
         SystemNumber , String , 88
         InstanceName , String , HDB88
         InstanceType , String , HANA Test
         Hostname , String , hdb88
         FullQualifiedHostname , String , hdb88.example.com
         IPAddress , String , 10.0.0.88
         SapVersionInfo , String , 749, patch 211, changelist 1754007
        *********************************************************
         SID , String , D90
         SystemNumber , String , 90
         InstanceName , String , HDB90
         InstanceType , String , HANA Test
         Hostname , String , hdb90
         FullQualifiedHostname , String , hdb90.example.com
         IPAddress , String , 10.0.0.90
         SapVersionInfo , String , 749, patch 211, changelist 1754007

    Examples:
        >>> type(sap_inst)
        <class 'insights.parsers.saphostctrl.SAPHostCtrlInstances'>
        >>> sap_inst[1]['SID']
        'D90'
        >>> sap_inst[0]['SapVersionInfo']
        '749, patch 211, changelist 1754007'
        >>> sap_inst[1]['InstanceType']
        'HANA Test'
        >>> sap_inst.types
        ['HDB']
        >>> sorted(sap_inst.sids)
        ['D89', 'D90']

    Attributes:
        instances (list): The list of instances found in the cluster output.
        sids (list): The list of SID found in the cluster output.
        types (list): The list of short instance types.  The short instance
            type is set as the characters of the `InstanceName` before the
            `SystemNumber`.

    Raises:
        SkipComponent: When input is empty.
        ParseException: When input cannot be parsed.
    """
    REQUIRED_DIRECTIVES = (
        'SID',
        'SystemNumber',
        'InstanceName',
        'InstanceType',
        'Hostname',
        'SapVersionInfo',
        'FullQualifiedHostname'
    )

    def parse_content(self, content):
        self.instances = []
        _sids = set()
        _types = set()

        inst = {}
        for line in (l.strip() for l in content):
            if line.startswith('******'):
                inst = {}
                self.append(inst)
                continue

            fields = [i.strip() for i in line.split(',', 2)]
            if len(fields) < 3:
                raise ParseException("Incorrect line: '{0}'".format(line))

            inst[fields[0]] = fields[2]

            if fields[0] == 'InstanceName':
                short_type = fields[2].strip('0123456789')
                # Back forward compatible: a short InstanceType was from the InstanceName
                if 'InstanceType' not in inst:
                    inst.update(dict(InstanceType=short_type))
                self.instances.append(fields[2])
                _types.add(short_type)
            _sids.add(fields[2]) if fields[0] == 'SID' else None

        if len(self) < 1:
            raise SkipComponent("Nothing need to parse")

        self.sids = list(_sids)
        self.types = list(_types)

    @property
    def data(self):
        """
        (list): List of dicts where keys are the lead name of each line and
            values are the string value.
        """
        return self



import pytest
import shlex
import sys

from insights.core.exceptions import CalledProcessError
from insights.util import subproc


def test_call():
    result = subproc.call('echo -n hello')
    assert result == 'hello'


def test_call_list_of_lists():
    cmd = "echo -n ' hello '"
    cmd = shlex.split(cmd)
    result = subproc.call([cmd, ["grep", "-F", "hello"]])
    assert "hello" in result


def test_call_timeout():
    # Timeouts don't work on OS X
    if sys.platform != "darwin":
        with pytest.raises(CalledProcessError):
            subproc.call('sleep 3', timeout=1)



"""
Custom datasources for candlepin broker.xml
"""
from insights.components.satellite import IsSatellite
from insights.core import ET
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_file
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by candlepin_broker datasources """

    candlepin_broker_input = simple_file("/etc/candlepin/broker.xml")
    """ Returns the contents of the file ``/etc/candlepin/broker.xml`` """


@datasource(LocalSpecs.candlepin_broker_input, HostContext, IsSatellite)
def candlepin_broker(broker):
    """
    This datasource provides the candlepn broker configuration information
    collected from ``/etc/candlepin/broker.xml``.

    Typical content of ``/etc/candlepin/broker.xml`` file is::

        <configuration xmlns="urn:activemq"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="urn:activemq /schema/artemis-configuration.xsd">
            <core xmlns="urn:activemq:core" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xsi:schemaLocation="urn:activemq:core ">
                <acceptors>
                    <acceptor name="in-vm">vm://0</acceptor>
                    <acceptor name="stomp">tcp://localhost:61613?protocols=STOMP;useEpoll=false;sslEnabled=true;trustStorePath=/etc/candlepin/certs/truststore;trustStorePassword=CDX9i3K5uPPBzcNtzz5tcycVf5PuXA5w;keyStorePath=/etc/candlepin/certs/keystore;keyStorePassword=4iBpTS45VZjFmVdNzRhRKNXtxbsH5Dij;needClientAuth=true</acceptor>
                </acceptors>
                <security-enabled>true</security-enabled>
            </core>
        </configuration>

    Note:
        This datasource may be executed using the following command:

        ``insights cat --no-header candlepin_broker``

    Returns:
        str: XML string after removeing sensitive information.

    Raises:
        SkipComponent: When the path does not exist or any exception occurs.
    """

    relative_path = '/etc/candlepin/broker.xml'
    try:
        content = broker[LocalSpecs.candlepin_broker_input].content
        if content:
            root = ET.fromstring('\n'.join(content))
            # remove namespace before save to avoid urgly search
            for node in list(root.iter()):
                prefix, has_namespace, postfix = node.tag.rpartition('}')
                if has_namespace:
                    node.tag = postfix
            # remove sensitive data
            core_ele = root.find('core')
            passwd_ele = core_ele.find('cluster-password')
            if passwd_ele is not None:
                core_ele.remove(passwd_ele)
            acc_ele = core_ele.find('acceptors')
            if acc_ele:
                core_ele.remove(acc_ele)
            return DatasourceProvider(
                content=[line for line in ET.tostring(root).decode('utf-8').splitlines() if line.strip()],
                relative_path=relative_path
            )
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent()



"""
GlusterPeerStatus - command ``gluster peer status``
===================================================
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs
from insights.specs import Specs

try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest


@parser(Specs.gluster_peer_status)
class GlusterPeerStatus(CommandParser):
    """Parse the output of ``gluster peer status``

    Typical output of ``gluster peer status`` command is::

        Number of Peers: 1

        Hostname: versegluster1.verse.loc
        Uuid: 86c0266b-c78c-4d0c-afe7-953dec143530
        State: Peer in Cluster (Connected)

    Examples:
        >>> output.status['peers']
        1
        >>> len(output.status.get('hosts', []))
        1
        >>> output.status.get('hosts', [])[0].get('Hostname')
        'versegluster1.verse.loc'

    Attributes:
        status (dict): A dict with keys ``peers`` and ``hosts``. For example::

            {'peers': 3,
             'hosts': [
                       {'Hostname': 'foo.com', 'State': 'Peer in Cluster (Connected)', 'Uuid': '86c0266b-c78c-4d0c-afe7-953dec143530'},
                       {'Hostname': 'example.com', 'State': 'Peer in Cluster (Connected)', 'Uuid': '3b4673e3-5e95-4c02-b9bb-2823483e067b'},
                       {'Hostname': 'bar.com', 'State': 'Peer in Cluster (Disconnected)', 'Uuid': '4673e3-5e95-4c02-b9bb-2823483e067bb3'}]
            }

    """
    def _grouper(self, iterable, n, fillvalue=None):
        "Collect data into fixed-length chunks or blocks"
        args = [iter(iterable)] * n
        return zip_longest(*args, fillvalue=fillvalue)

    def parse_content(self, content):
        if not content or "Connection failed" in content[0]:
            raise SkipComponent("No data.")

        self.status = {'peers': 0, 'hosts': []}
        self.status['peers'] = int(content[0].split(':')[-1].strip())
        for group in list(self._grouper(list(filter(lambda x: x != '', content[2:])), 3)):
            self.status['hosts'].append(split_kv_pairs(group, split_on=':'))



"""
NeutronServerLog - file ``/var/log/neutron/server.log``
=======================================================
"""
from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.neutron_server_log)
class NeutronServerLog(LogFileOutput):
    '''
    Read the ``/var/log/neutron/server.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput` for
        more usage information

    Sample log file::

        2016-09-13 05:56:45.155 30586 WARNING keystonemiddleware.auth_token [-] Identity response: {"error": {"message": "Could not find token: b45405915eb44e608885f894028d37b9", "code": 404, "title": "Not Found"}}
        2016-09-13 05:56:45.156 30586 WARNING keystonemiddleware.auth_token [-] Authorization failed for token
        2016-09-13 06:06:45.884 30588 WARNING keystonemiddleware.auth_token [-] Authorization failed for token
        2016-09-13 06:06:45.886 30588 WARNING keystonemiddleware.auth_token [-] Identity response: {"error": {"message": "Could not find token: fd482ef0ba1144bf944a0a6c2badcdf8", "code": 404, "title": "Not Found"}}
        2016-09-13 06:06:45.887 30588 WARNING keystonemiddleware.auth_token [-] Authorization failed for token
        2016-09-13 06:06:46.131 30586 WARNING keystonemiddleware.auth_token [-] Authorization failed for token
        2016-09-13 06:06:46.131 30586 WARNING keystonemiddleware.auth_token [-] Identity response: {"error": {"message": "Could not find token: bc029dbe33f84fbcb67ef7d592458e60", "code": 404, "title": "Not Found"}}
        2016-09-13 06:06:46.132 30586 WARNING keystonemiddleware.auth_token [-] Authorization failed for token

    Examples:
        >>> neutron_log = shared[NeutronServerLog]
        >>> neutron_log.get('Authorization')[0]['raw_message']
        '2016-09-13 05:56:45.156 30586 WARNING keystonemiddleware.auth_token [-] Authorization failed for token'
        >>> len(list(neutron_log.get_after(datetime.datetime(2016, 9, 13, 6, 0, 0))))
        6
        >>> neutron_log.get_after(datetime.datetime(2016, 9, 13, 6, 0, 0))[0]['raw_message']
        '2016-09-13 06:06:45.884 30588 WARNING keystonemiddleware.auth_token [-] Authorization failed for token'
    '''
    pass



"""
BlacklistedSpecs - File ``blacklisted_specs``
=============================================
"""
from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.blacklisted_specs)
class BlacklistedSpecs(JSONParser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.client_metadata.BlacklistedSpecs` instead.

    Parses the "blacklisted_specs" or "blacklisted_specs.txt" file generated
    on archive creation.

    Typical output::
        "{"specs": ["dmesg", "fstab"]}"

    Attributes:
        specs (list): List of blacklisted specs.

    Examples:
        >>> type(specs)
        <class 'insights.parsers.blacklisted.BlacklistedSpecs'>
        >>> result = ['dmesg', 'fstab']
        >>> specs.specs == result
        True
    """
    def __init__(self, *args, **kwargs):
        deprecated(
            BlacklistedSpecs,
            "Please use insights.parsers.client_metadata.BlacklistedSpecs instead.",
            "3.3.25"
        )
        super(BlacklistedSpecs, self).__init__(*args, **kwargs)

    @property
    def specs(self):
        return self.data['specs']



import doctest
import pytest
from insights.parsers import cgroups
from insights.parsers.cgroups import Cgroups
from insights.tests import context_wrap

cgroups_content = """
#subsys_name	hierarchy	num_cgroups	enabled
cpuset	10	48	1
cpu	2	232	1
cpuacct	2	232	1
memory	5	232	1
devices	6	232	1
freezer	3	48	1
net_cls	4	48	1
blkio	9	232	1
perf_event	8	48	1
hugetlb	11	48	1
pids	7	232	1
net_prio	4	48	1
""".strip()


def test_cgroups():
    context = context_wrap(cgroups_content)
    i_cgroups = Cgroups(context)
    assert i_cgroups.get_num_cgroups("memory") == 232
    assert i_cgroups.is_subsys_enabled("memory") is True
    assert i_cgroups.subsystems["memory"]["enabled"] == "1"
    with pytest.raises(KeyError) as ke:
        i_cgroups.get_num_cgroups("Wrong_memory")
    assert "wrong subsys_name" in str(ke)


def test_cgroup_documentation():
    env = {
        'i_cgroups': Cgroups(context_wrap(cgroups_content))
    }
    failed, total = doctest.testmod(cgroups, globs=env)
    assert failed == 0



import doctest

from insights.parsers import cloud_cfg
from insights.tests import context_wrap
from insights.tests.parsers import skip_component_check


CONFIG_1 = """
{"ssh_deletekeys": 1, "network": {"config": "disabled"}}
"""

CONFIG_2 = """
{"ssh_deletekeys": 1, "network": {"version": 1, "config": [{"type": "physical", "name": "eth0", "subnets": [{"type": "dhcp"}, {"type": "dhcp6"}]}]}, "debug": {"output": "/var/log/cloud-init-debug.log", "verbose": true}}
"""


def test_cloud_cfg():
    result = cloud_cfg.CloudCfg(context_wrap(CONFIG_1))
    assert result.data['network'] == {'config': 'disabled'}

    result = cloud_cfg.CloudCfg(context_wrap(CONFIG_2))
    assert result.data['network']['config'][0]['name'] == 'eth0'


def test_cloud_cfg_empty():
    assert 'There is no data' in skip_component_check(cloud_cfg.CloudCfg)


def test_doc_examples():
    env = {
        'cloud_cfg': cloud_cfg.CloudCfg(context_wrap(CONFIG_2)),
    }
    failed, total = doctest.testmod(cloud_cfg, globs=env)
    assert failed == 0



"""
up2date Logs -  Files ``/var/log/up2date``
==========================================

Modules for parsing the content of log file ``/var/log/up2date`` in sosreport archives of RHEL.

"""
from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.up2date_log)
class Up2dateLog(LogFileOutput):
    """
    Class for parsing the log file: ``/var/log/up2date``.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`

    Example content of ``/var/log/up2date`` command is::

        [Thu Feb  1 02:46:25 2018] rhn_register updateLoginInfo() login info
        [Thu Feb  1 02:46:35 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #1
        [Thu Feb  1 02:46:40 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #2
        [Thu Feb  1 02:46:45 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #3
        [Thu Feb  1 02:46:50 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #4
        [Thu Feb  1 02:46:55 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #5

        ...

    Examples:
        >>> ulog.get('Temporary failure in name resolution')[0]['raw_message']
        "[Thu Feb  1 02:46:35 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #1"
    """

    time_format = '%a %b %d %H:%M:%S %Y'



"""
Custom datasources for aws information
"""
from insights.components.cloud_provider import IsAWS
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by aws datasources """
    aws_imdsv2_token = simple_command(
        '/usr/bin/curl -s -X PUT "http://169.254.169.254/latest/api/token" -H "X-aws-ec2-metadata-token-ttl-seconds: 60" --connect-timeout 5',
        deps=[IsAWS]
    )


@datasource(LocalSpecs.aws_imdsv2_token, HostContext)
def aws_imdsv2_token(broker):
    """
    This datasource provides a session token for use by other specs to collect
    metadata information on AWS EC2 nodes with IMDSv2 support..

    Typical output of the input spec, which is also the output of this datasource::

        AQAEABcCFaLcKRfXhLV9_ezugiVzra-qMBoPbdWGLrbdfqSLEJzP8w==

    Returns:
        str: String that is the actual session token to be used in other commands

    Raises:
        SkipComponent: When an error occurs or no token is generated
    """
    try:
        token = broker[LocalSpecs.aws_imdsv2_token].content[0].strip()
        if token:
            return str(token)
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent



"""
ProcStat - File ``/proc/stat``
==============================

This parser reads the content of ``/proc/stat``.
"""
from insights.core import CommandParser, LegacyItemAccess
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.proc_stat)
class ProcStat(CommandParser, LegacyItemAccess):
    """
    Class ``ProcStat`` parses the content of the ``/proc/stat``.

    Attributes:
        cpu_percentage (string):     The CPU usage percentage since boot.
        intr_total (int):            The total of all interrupts serviced including unnumbered
                                     architecture specific interrupts.
        ctxt (int):                  The number of context switches that the system under went.
        btime (string):              boot time, in seconds since the Epoch, 1970-01-01
                                     00:00:00 +0000 (UTC).
        processes (int):             Number of forks since boot.
        procs_running (int):         Number of processes in runnable state. (Linux 2.5.45
                                     onward.)
        procs_blocked (int):         Number of processes blocked waiting for I/O to complete.
                                     (Linux 2.5.45 onward.)
        softirq_total (int) :        The total of all softirqs and each subsequent column is
                                     the total for particular softirq.
                                     (Linux 2.6.31 onward.)

    A small sample of the content of this file looks like::

        cpu  32270961 89036 23647730 1073132344 1140756 0 1522035 18738206 0 0
        cpu0 3547155 11248 2563031 135342787 113432 0 199615 2199379 0 0
        cpu1 4660934 10954 3248126 132271933 120282 0 279870 2660186 0 0
        cpu2 4421035 10729 3306081 132914999 126705 0 194141 2505565 0 0
        cpu3 4224551 10633 3139695 133634676 121035 0 181213 2380738 0 0
        cpu4 3985452 11151 2946570 134064686 205568 0 165839 2478471 0 0
        cpu5 3914912 11396 2896447 134635676 117341 0 164794 2260011 0 0
        cpu6 3802544 11418 2817453 134878674 222855 0 182738 2150276 0 0
        cpu7 3714375 11503 2730323 135388911 113534 0 153821 2103576 0 0
        intr 21359029 22 106 0 0 0 0 3 0 1 0 16 155 357 0 0 671261 0 0 0 0 0 0 0 0 0 0 0 32223 0 4699385 2 0 8 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
        ctxt 17852681
        btime 1542179825
        processes 19212
        procs_running 1
        procs_blocked 0
        softirq 11867930 1 3501158 6 4705528 368244 0 79 2021509 0 1271405



    Examples:
        >>> type(proc_stat)
        <class 'insights.parsers.proc_stat.ProcStat'>
        >>> proc_stat.cpu_percentage
        '6.73%'
        >>> proc_stat.btime
        '1542179825'
        >>> proc_stat.ctxt
        17852681
        >>> proc_stat.softirq_total
        11867930
        >>> proc_stat.intr_total
        21359029
        >>> proc_stat.processes
        19212
        >>> proc_stat.procs_running
        1
        >>> proc_stat.procs_blocked
        0
    """

    def parse_content(self, content):
        self.data = {}
        for line in get_active_lines(content):
            key, value = line.split(None, 1)
            self.data.update({key: value})

        """
        The following code is used to calculate the cpu usage percentage.
        The output of a cpu line could contain the following information according to different kernel version. Earlier
        versions do not have guest, guestnice for this two are involving for virtual machine in some later version.
        The cpu usage percentage is calculated by idle divide total.
        - user:   the number of jiffies (1/100 of a second for x86 systems) that the system has been in user mode
        - nice:   the number of jiffies that the system has been in user mode with low priority (nice)
        - system: the number of jiffies that the system has been in system mode
        - idle:   the number of jiffies that the system has been in idle task
        - iowait: the number of jiffies that the system has been in I/O wait
        - irq:    the number of jiffies that the system has been in servicing interrupts
        - softirq:the number of jiffies that the system has been in servicing softirqs
        - steal:  the number of jiffies that the system involuntary wait
        - guest:  the number of jiffies that the system is running a normal guest
        - guestnice: the number of jiffies that the system is running a niced guest
        """
        self.cpu_percentage = None
        if 'cpu' in self.data:
            cpu_list = list(map(lambda x: int(x), self.data['cpu'].split()))
            if len(cpu_list) < 7:
                raise ParseException("Error: Invalid cpu length", self.data['cpu'])
            cpu_total = float(sum(cpu_list))
            cpu_idle = float(cpu_list[3])
            cpu_pct = (cpu_total - cpu_idle) * 100 / cpu_total
            self.cpu_percentage = "{0:.2f}".format(cpu_pct) + '%'

        # Get the total number of hard interrupts
        self.intr_total = int(self.data['intr'].split(None, 1)[0]) if 'intr' in self.data else None

        # Get the total number of soft interrupts
        self.softirq_total = int(self.data['softirq'].split(None, 1)[0]) if 'softirq' in self.data else None

        self.ctxt = int(self.data['ctxt']) if 'ctxt' in self.data else None

        self.btime = self.data['btime'] if 'btime' in self.data else None

        self.processes = int(self.data['processes']) if 'processes' in self.data else None

        self.procs_running = int(self.data['procs_running']) if 'procs_running' in self.data else None

        self.procs_blocked = int(self.data['procs_blocked']) if 'procs_blocked' in self.data else None



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import cups_ppd
from insights.parsers.cups_ppd import CupsPpd
from insights.tests import context_wrap


CUPS_PPD = """
*PPD-Adobe: "4.3"
*% Copyright 2007-2014 by Apple Inc.
*% test: this line is used to check comment
*FormatVersion: "4.3"
*FileVersion: "2.2"
*LanguageVersion: English
*LanguageEncoding: ISOLatin1
*PCFileName: "ippeve.ppd"
*Manufacturer: "Canon"
*ModelName: "iR-ADV C3525/3530 PPD"
*Product: "(iR-ADV C3525/3530 PPD)"
*NickName: "iR-ADV C3525/3530 PPD"
*ShortNickName: "iR-ADV C3525/3530 PPD"
*cupsFilter2: "application/vnd.cups-pdf application/pdf 10 -"
*cupsFilter2: "application/vnd.cups-postscript application/postscript 10 -"
""".strip()

CUPS_PPD_INVALID1 = '''
'''.strip()

CUPS_PPD_INVALID2 = '''
ShortNickName
'''.strip()


def test_cups_ppd():
    cups_ppd_result = CupsPpd(context_wrap(CUPS_PPD, path='/etc/cups/ppd/test_printer1.ppd'))
    assert cups_ppd_result["PCFileName"] == '"ippeve.ppd"'
    assert cups_ppd_result["cupsFilter2"] == ['"application/vnd.cups-pdf application/pdf 10 -"', '"application/vnd.cups-postscript application/postscript 10 -"']
    assert "test" not in cups_ppd_result

    with pytest.raises(SkipComponent) as exc:
        CupsPpd(context_wrap(CUPS_PPD_INVALID1, path='/etc/cups/ppd/test_printer1.ppd'))
    assert 'No Valid Configuration' in str(exc)

    with pytest.raises(SkipComponent) as exc:
        CupsPpd(context_wrap(CUPS_PPD_INVALID2, path='/etc/cups/ppd/test_printer1.ppd'))
    assert 'No Valid Configuration' in str(exc)


def test_cups_ppd_documentation():
    env = {
        'cups_ppd': CupsPpd(context_wrap(CUPS_PPD,
            path='/etc/cups/ppd/test_printer1.ppd'))
    }
    failed, total = doctest.testmod(cups_ppd, globs=env)
    assert failed == 0
